package de.ariva.six.dochub.restclient;

import de.ariva.six.dochub.restclient.entities.ArchiveSearchRequest;
import de.ariva.six.dochub.restclient.entities.ArchiveSearchResponse;
import de.ariva.six.dochub.restclient.entities.AuthenticationRequest;
import de.ariva.six.dochub.restclient.entities.AuthenticationResponse;
import de.ariva.six.dochub.restclient.entities.CustomDataRequest;
import de.ariva.six.dochub.restclient.entities.SearchRequest;
import de.ariva.six.dochub.restclient.entities.SearchResponse;
import lombok.extern.slf4j.Slf4j;
import org.springframework.core.io.Resource;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.lang.Nullable;
import org.springframework.stereotype.Component;
import org.springframework.web.client.RestTemplate;

import java.io.IOException;
import java.io.InputStream;
import java.util.Collections;
import java.util.Map;
import java.util.Objects;

/**
 * This class contains example methods that use the DocHub REST API.
 */
@Component
@Slf4j
public class DemoRestClient {
    private static final String URL_BASE = "https://test.six-dochub.com";

    /**
     * Sends an authentication request to the DocHub REST API.
     *
     * @param username the username
     * @param password the password
     * @return the token
     */
    public String getAuthenticationToken(final String username, final String password) {
        /* add proxy settings like this if needed:
        System.getProperties().put("https.proxySet", "true");
        System.getProperties().put("https.proxyHost", "<proxy-host>");
        System.getProperties().put("https.proxyPort", "<proxy-port>");
        */

        final AuthenticationRequest request = AuthenticationRequest
                .builder()
                .username(username)
                .password(password)
                .build();

        final RestTemplate restTemplate = new RestTemplate();
        final ResponseEntity<AuthenticationResponse> authenticationResponseResponseEntity = restTemplate.postForEntity(
                URL_BASE + "/api/rest/v1/authentication/authorization-tokens",
                request,
                AuthenticationResponse.class
        );

        final AuthenticationResponse body = authenticationResponseResponseEntity.getBody();
        return Objects.requireNonNull(body).getToken();
    }

    /**
     * Sends a content-requests request to the DocHub API with archive=false.
     *
     * @param token the authentication token
     * @param isinValue the ISIN to look for
     * @return the response entity containing the PDF file data and response headers
     */
    public ResponseEntity<Resource> getPib(final String token, final String isinValue) {
        final CustomDataRequest request = CustomDataRequest.builder().build();

        final HttpHeaders httpHeaders = new HttpHeaders();
        httpHeaders.setBearerAuth(token);
        httpHeaders.setAccept(Collections.singletonList(MediaType.APPLICATION_PDF));

        final HttpEntity<CustomDataRequest> httpEntity = new HttpEntity<>(request, httpHeaders);

        final RestTemplate restTemplate = new RestTemplate();

        return restTemplate.exchange(
                String.format(
                        URL_BASE + "/api/rest/v1/instruments/isin/%s/pib/content-requests?archive=false",
                        isinValue
                ),
                HttpMethod.POST,
                httpEntity,
                Resource.class
        );
    }

    /**
     * Sends a content-requests request to the DocHub API with archive=true.
     *
     * @param token the authentication token
     * @param isinValue the ISIN to look for
     * @return the response entity containing the PDF file data and response headers
     */
    public ResponseEntity<Resource> getPibAndArchive(
            final String token,
            final String isinValue
    ) {
        final CustomDataRequest request = CustomDataRequest.builder().build();

        final HttpHeaders httpHeaders = new HttpHeaders();
        httpHeaders.setBearerAuth(token);
        httpHeaders.setAccept(Collections.singletonList(MediaType.APPLICATION_PDF));

        final HttpEntity<CustomDataRequest> httpEntity = new HttpEntity<>(request, httpHeaders);

        final RestTemplate restTemplate = new RestTemplate();

        return restTemplate.exchange(
                String.format(
                        URL_BASE + "/api/rest/v1/instruments/isin/%s/pib/content-requests?archive=true",
                        isinValue
                ),
                HttpMethod.POST,
                httpEntity,
                Resource.class
        );
    }

    /**
     * Sends an archive content-requests request to the DocHub API.
     * This retrieves the PDF file data of an archive entry.
     *
     * @param token the authentication token
     * @param archiveId the archive ID to look for
     * @param customData optional custom data header value
     * @return the PDF file data of the archived document
     * @throws IOException if something goes wrong with the remote API
     */
    public InputStream getDocumentFromArchive(
            final String token,
            final String archiveId,
            @Nullable final String customData
    ) throws IOException {
        final CustomDataRequest request = CustomDataRequest.builder().build();

        final HttpHeaders httpHeaders = new HttpHeaders();
        httpHeaders.setBearerAuth(token);
        httpHeaders.setAccept(Collections.singletonList(MediaType.APPLICATION_PDF));
        if (customData != null) {
            httpHeaders.add("x-custom-data", customData);
        }

        final HttpEntity<CustomDataRequest> httpEntity = new HttpEntity<>(request, httpHeaders);

        final RestTemplate restTemplate = new RestTemplate();

        return Objects.requireNonNull(restTemplate.exchange(
                String.format(
                        URL_BASE + "/api/rest/v1/documents/archived/%s/content-requests",
                        archiveId
                ),
                HttpMethod.POST,
                httpEntity,
                Resource.class
        ).getBody()).getInputStream();
    }

    public ResponseEntity<Resource> getDocument(
            final String token,
            final String documentId,
            @Nullable final String customData
    ) {
        final CustomDataRequest request = CustomDataRequest.builder().build();

        final HttpHeaders httpHeaders = new HttpHeaders();
        httpHeaders.setBearerAuth(token);
        httpHeaders.setAccept(Collections.singletonList(MediaType.APPLICATION_PDF));
        if (customData != null) {
            httpHeaders.add("x-custom-data", customData);
        }

        final HttpEntity<CustomDataRequest> httpEntity = new HttpEntity<>(request, httpHeaders);

        final RestTemplate restTemplate = new RestTemplate();

        return Objects.requireNonNull(restTemplate.exchange(
                String.format(
                        URL_BASE + "/api/rest/v1/documents/latest/%s/content-requests",
                        documentId
                ),
                HttpMethod.POST,
                httpEntity,
                Resource.class
        ));
    }

    public SearchResponse searchForDocuments(final String token) {
        final HttpHeaders httpHeaders = new HttpHeaders();
        httpHeaders.setBearerAuth(token);

        final HttpEntity<Map<?, ?>> httpEntity = new HttpEntity<>(Collections.emptyMap(), httpHeaders);

        final RestTemplate restTemplate = new RestTemplate();

        return restTemplate.exchange(
                URL_BASE + "/api/rest/v2/documents/latest/search-requests",
                HttpMethod.POST,
                httpEntity,
                SearchResponse.class
        ).getBody();
    }

    public SearchResponse searchForDocumentsWithParameters(final String token, final SearchRequest searchRequest) {
        final HttpHeaders httpHeaders = new HttpHeaders();
        httpHeaders.setBearerAuth(token);

        final HttpEntity<SearchRequest> httpEntity = new HttpEntity<>(searchRequest, httpHeaders);

        final RestTemplate restTemplate = new RestTemplate();

        return restTemplate.exchange(
                URL_BASE + "/api/rest/v2/documents/latest/search-requests",
                HttpMethod.POST,
                httpEntity,
                SearchResponse.class
        ).getBody();
    }


    public ArchiveSearchResponse searchForArchivedDocuments(final String token) {
        final HttpHeaders httpHeaders = new HttpHeaders();
        httpHeaders.setBearerAuth(token);

        final HttpEntity<Map<?, ?>> httpEntity = new HttpEntity<>(Collections.emptyMap(), httpHeaders);

        final RestTemplate restTemplate = new RestTemplate();

        return restTemplate.exchange(
                URL_BASE + "/api/rest/v2/documents/archived/search-requests",
                HttpMethod.POST,
                httpEntity,
                ArchiveSearchResponse.class
        ).getBody();
    }

    public ArchiveSearchResponse searchForArchivedDocumentsWithParameters(
            final String token,
            final ArchiveSearchRequest archiveSearchRequest
    ) {
        final HttpHeaders httpHeaders = new HttpHeaders();
        httpHeaders.setBearerAuth(token);

        final HttpEntity<ArchiveSearchRequest> httpEntity = new HttpEntity<>(archiveSearchRequest, httpHeaders);

        final RestTemplate restTemplate = new RestTemplate();

        return restTemplate.exchange(
                URL_BASE + "/api/rest/v2/documents/archived/search-requests",
                HttpMethod.POST,
                httpEntity,
                ArchiveSearchResponse.class
        ).getBody();
    }
}
