package de.ariva.six.dochub.restclient;

import de.ariva.six.dochub.restclient.entities.ArchiveSearchRequest;
import de.ariva.six.dochub.restclient.entities.ArchiveSearchResponse;
import de.ariva.six.dochub.restclient.entities.GeneralIdentifier;
import de.ariva.six.dochub.restclient.entities.SearchRequest;
import de.ariva.six.dochub.restclient.entities.SearchResponse;
import lombok.extern.slf4j.Slf4j;
import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.core.io.Resource;
import org.springframework.http.ResponseEntity;
import org.springframework.web.client.HttpClientErrorException;

import java.io.IOException;
import java.io.InputStream;
import java.util.List;

import static org.assertj.core.api.Assertions.assertThat;
import static org.assertj.core.api.Assertions.assertThatThrownBy;

@SpringBootTest
@Slf4j
class DemoRestClientIntegrationTest {
    private static final String USERNAME = "REPLACE THIS WITH YOUR USERNAME";
    private static final String PASSWORD = "REPLACE THIS WITH YOUR PASSWORD";

    private static final String WRONG_PASSWORD = "wrong password";

    private static final String EXAMPLE_ISIN = "CH0029956262";

    @Autowired
    private DemoRestClient demoRestClient;

    @Test
    void login() {
        log.info("Trying to log in");
        final String token = demoRestClient.getAuthenticationToken(USERNAME, PASSWORD);
        log.info("Logged in, token: {}", token);

        assertThat(token).isNotEmpty();
    }

    @Test
    void loginWithWrongPassword() {
        assertThatThrownBy(
                () -> demoRestClient.getAuthenticationToken(USERNAME, WRONG_PASSWORD)
        ).isInstanceOf(HttpClientErrorException.class).hasMessageContaining("1001");
    }

    @Test
    @Disabled("This test case requires a special permission")
    void getPib() throws IOException {
        final String token = demoRestClient.getAuthenticationToken(USERNAME, PASSWORD);
        final InputStream stream = demoRestClient.getPib(token, EXAMPLE_ISIN).getBody().getInputStream();
        final byte[] pdf = stream.readAllBytes();
        log.info("Got PDF data with length {}", pdf.length);

        assertThat(pdf).isNotEmpty();
    }

    @Test
    @Disabled("This test case requires a special permission")
    void getPibWithArchiveTrueAndThenRetrieveFromArchive() throws IOException {
        final String token = demoRestClient.getAuthenticationToken(USERNAME, PASSWORD);
        final ResponseEntity<Resource> responseEntity = demoRestClient.getPibAndArchive(token, EXAMPLE_ISIN);
        final byte[] pdf = responseEntity.getBody().getInputStream().readAllBytes();
        log.info("Got PDF data with length {}", pdf.length);
        final String archiveId = responseEntity.getHeaders().getFirst("X-Archive-ID");
        log.info("Archive ID is {}", archiveId);

        final InputStream archivePdfInputStream = demoRestClient.getDocumentFromArchive(
                token,
                archiveId,
                null
        );
        final byte[] archivePdf = archivePdfInputStream.readAllBytes();
        log.info("Got archive PDF data with length {}", archivePdf.length);

        assertThat(pdf).hasSameSizeAs(archivePdf);
    }

    @Test
    void getArchivedDocumentByInvalidArchiveId() {
        final String token = demoRestClient.getAuthenticationToken(USERNAME, PASSWORD);
        assertThatThrownBy(() -> demoRestClient.getDocumentFromArchive(
                token,
                "123abc",
                null)
        ).isInstanceOf(HttpClientErrorException.class);
    }

    @Test
    void getArchivedDocumentUsingInvalidToken() {
        assertThatThrownBy(
                () -> demoRestClient.getDocumentFromArchive("123", "123abc", null)
        ).isInstanceOf(HttpClientErrorException.Unauthorized.class);
    }

    @Test
    void searchForDocuments() {
        final String token = demoRestClient.getAuthenticationToken(USERNAME, PASSWORD);
        final SearchResponse searchResponse = demoRestClient.searchForDocuments(token);
        log.info("Search response: {}", searchResponse);

        assertThat(searchResponse.getTotalHits()).isNotZero();
        assertThat(searchResponse.getHits()).isNotEmpty();
    }

    @Test
    void searchForArchivedDocuments() {
        final String token = demoRestClient.getAuthenticationToken(USERNAME, PASSWORD);
        final ArchiveSearchResponse searchResponse = demoRestClient.searchForArchivedDocuments(token);
        log.info("Archive search response: {}", searchResponse);

        assertThat(searchResponse.getTotalHits()).isNotZero();
        assertThat(searchResponse.getHits()).isNotEmpty();
    }

    @Test
    void searchForArchivedDocumentsByLanguageAndJurisdiction() {
        final String token = demoRestClient.getAuthenticationToken(USERNAME, PASSWORD);

        final ArchiveSearchRequest archiveSearchRequest = ArchiveSearchRequest
                .builder()
                .languageCode("de")
                .jurisdictionCode("DE")
                .build();

        final ArchiveSearchResponse searchResponse
                = demoRestClient.searchForArchivedDocumentsWithParameters(token, archiveSearchRequest);
        log.info("Archive search response: {}", searchResponse);

        assertThat(searchResponse.getTotalHits()).isNotZero();
        assertThat(searchResponse.getHits()).isNotEmpty();
    }

    @Test
    void findDocumentsByIsinAndLanguageAndJurisdiction() {
        final String token = demoRestClient.getAuthenticationToken(USERNAME, PASSWORD);

        final SearchRequest request = SearchRequest.builder().generalIdentifier(
                GeneralIdentifier.builder().type(1).value(EXAMPLE_ISIN).build()
        ).languageCodes(List.of("it", "de", "fr", "en")).jurisdictionCodes(List.of("DE", "CH")).build();

        final SearchResponse searchResponse = demoRestClient.searchForDocumentsWithParameters(token, request);
        log.info("Search response: {}", searchResponse);

        assertThat(searchResponse.getTotalHits()).isNotZero();
        assertThat(searchResponse.getHits()).isNotEmpty();
    }

    @Test
    void findDocumentsByIsinAndDocumentTypesAndDownloadPdfAndDownloadFromArchive() throws IOException {
        final String token = demoRestClient.getAuthenticationToken(USERNAME, PASSWORD);

        final SearchRequest request = SearchRequest.builder().generalIdentifier(
                GeneralIdentifier
                        .builder()
                        .type(1)
                        .value(EXAMPLE_ISIN)
                        .build()
        ).documentTypes(List.of(1, 3)).build();

        final SearchResponse searchResponse = demoRestClient.searchForDocumentsWithParameters(token, request);
        log.info("Search response: {}", searchResponse);

        assertThat(searchResponse.getTotalHits()).isNotZero();
        assertThat(searchResponse.getHits()).isNotEmpty();

        final ResponseEntity<Resource> responseEntity = demoRestClient.getDocument(
                token,
                searchResponse.getHits().get(0).getDocumentID(),
                null
        );
        final String archiveId = responseEntity.getHeaders().getFirst("X-Archive-ID");
        assertThat(archiveId).isNotBlank();
        assertThat(responseEntity.getBody().getInputStream()).isNotEmpty();
        final byte[] pdf = responseEntity.getBody().getInputStream().readAllBytes();

        final InputStream inputStreamFromArchive = demoRestClient.getDocumentFromArchive(
                token,
                archiveId,
                null
        );
        final byte[] pdfFromArchive = inputStreamFromArchive.readAllBytes();
        assertThat(pdf).isEqualTo(pdfFromArchive);
    }

    @Test
    void findDocumentsByIsinAndDocumentTypesAndDownloadPdfWithCustomDataAndDownloadFromArchiveWithCustomData()
    throws IOException {
        final String token = demoRestClient.getAuthenticationToken(USERNAME, PASSWORD);

        final SearchRequest request = SearchRequest.builder().generalIdentifier(
                GeneralIdentifier
                        .builder()
                        .type(1)
                        .value(EXAMPLE_ISIN)
                        .build()
        ).documentTypes(List.of(1, 3)).build();

        final SearchResponse searchResponse = demoRestClient.searchForDocumentsWithParameters(token, request);
        log.info("Search response: {}", searchResponse);

        assertThat(searchResponse.getTotalHits()).isNotZero();
        assertThat(searchResponse.getHits()).isNotEmpty();

        final ResponseEntity<Resource> responseEntity = demoRestClient.getDocument(
                token,
                searchResponse.getHits().get(0).getDocumentID(),
                "custom data abcdefg"
        );
        final String archiveId = responseEntity.getHeaders().getFirst("X-Archive-ID");
        assertThat(archiveId).isNotBlank();
        assertThat(responseEntity.getBody().getInputStream()).isNotEmpty();
        final byte[] pdf = responseEntity.getBody().getInputStream().readAllBytes();

        final InputStream inputStreamFromArchive = demoRestClient.getDocumentFromArchive(
                token,
                archiveId,
                "custom data abcdefg"
        );
        final byte[] pdfFromArchive = inputStreamFromArchive.readAllBytes();
        assertThat(pdf).isEqualTo(pdfFromArchive);
    }
}
